from aiohttp import ClientSession
from aiohttp_socks import ProxyType, ProxyConnector, ChainProxyConnector, ProxyError, \
	ProxyConnectionError, ProxyTimeoutError
from mo_dots import to_data
from random import randrange
import requests

proxies, one_hot = [], []
has_init = False


class ActError(Exception):
	""" General ActProxy exception. ActProxy also exports aiohttp[socks] exceptions: ProxyError, ProxyConnectionError,
	ProxyTimeoutError"""

	def __init__(self, message):
		self.message = message
		super().__init__(self.message)


def act_parse_json(proxy_items):
	"""
	Parse the proxy list JSON supplied by ActProxy API.
		:param proxy_items: List of proxy lines returned from ActProxy API.
		:return: List of dicts containing proxies.
	"""
	global one_hot, proxies
	for line in proxy_items:
		line_spl = line.split(';')
		host_spl = line_spl[0].split(':')
		proxies.append({
			'host': host_spl[0],
			'port': host_spl[1],
			'username': line_spl[1],
			'password': line_spl[2]
		})
	one_hot = [0 for p in proxies]
	return proxies


def init(api_key, output_format='json', get_userpass=True):
	"""
	Synchronously initialize ActProxy API & return proxies from account.
		:param api_key: ActProxy.com API key.
		:param output_format: 'json' or 'raw'; must be 'json' to use connectors.
		:param get_userpass: Include usernames & passwords in results? Must be True to use connectors.
		:return: A mo-dots object (Eg: proxies[0].username or proxies[0]['username']) in the case of 'json' as
				output_format. A Str in case of 'raw' as output_format.
	"""
	global proxies, one_hot, has_init
	userpass = 'true' if get_userpass else 'false'
	format_uri = '?format=json' if output_format == 'json' else ''
	api_url = f'https://actproxy.com/proxy-api/{api_key}?format={format_uri}&userpass={userpass}'
	resp = requests.get(api_url)
	if resp.status_code == 200:
		if output_format == 'json':
			proxy_items = resp.json()
			proxies = act_parse_json(proxy_items)
			has_init = True
			return to_data(proxies) if len(proxies) else None
		else:
			return resp.text
	else:
		raise ActError(f"HTTP error {resp.status_code} contacting ActProxy.")


async def aioinit(api_key, output_format='json', get_userpass=True):
	"""
	Asynchronously initialize ActProxy API & return proxies from account.
		:param api_key: ActProxy.com API key.
		:param output_format: 'json' or 'raw'; must be 'json' to use connectors.
		:param get_userpass: Include usernames & passwords in results? Must be True to use connectors.
		:return: A mo-dots object (Eg: proxies[0].username or proxies[0]['username']) in the case of 'json' as
				output_format. A Str in case of 'raw' as output_format.
	"""
	global proxies, one_hot, has_init
	has_init = True
	userpass = 'true' if get_userpass else 'false'
	async with ClientSession() as session:
		api_url = f'https://actproxy.com/proxy-api/{api_key}?format={output_format}&userpass={userpass}'
		async with session.get(api_url) as resp:
			if resp.status == 200:
				proxy_items = await resp.json(content_type=None)
				proxies = act_parse_json(proxy_items)
				return to_data(proxies) if len(proxies) else None
			else:
				raise ActError(f"HTTP error {resp.status} contacting ActProxy.")


def rotate(protocol='socks5'):
	"""
	Get the next proxy in the one-hot rotation for use with the requests[socks] package after having once run
	actproxy.init(output_format='json').
		:param protocol: 'socks5' or 'http'; must correspond to your ActProxy proxies' type.
		:return: Dict formatted for use in the requets[socks] package's proxies= parameter.
	"""
	proxy = one_hot_proxy()
	requests_proxy = {
		'http': f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}',
		'https': f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}'
	}
	return requests_proxy


def aiohttp_rotate(protocol='socks5'):
	"""
	Get an aiohttp connector that uses the next proxy in the one-hot rotation after having once run
	actproxy.aioinit(output_format='json').
		:param protocol: 'socks5' or 'http'; must correspond to your ActProxy proxies' type.
		:return: An aiohttp connector set to use the next proxy.
	"""
	proxy = one_hot_proxy()
	return ProxyConnector.from_url(f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}')


def random_proxy(protocol='socks5'):
	"""
	Get a random proxy from your account after having once run actproxy.init(output_format='json') or
	actproxy.aioinit(output_format='json')
		:param protocol: 'socks5' or 'http'; must be the same as your ActProxy proxies.
		:return: A random proxy mo-dots (Dict-like) object.
	"""
	global proxies
	last_prox = len(proxies) - 1
	rand_prox = randrange(0, last_prox)
	proxy = proxies[rand_prox]
	requests_proxy = {
		'http': f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}',
		'https': f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}'
	}
	return requests_proxy


def aiohttp_random(protocol='socks5'):
	"""
	Get a random aiohttp connector after having once run actproxy.aioinit(output_format='json')
		:param protocol: 'socks5' or 'http'; must be the same as your ActProxy proxies.
		:return:
	"""
	proxy = random_proxy()
	return ProxyConnector.from_url(f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}')


def one_hot_proxy():
	"""
	Get the next proxy in the one-hot rotation and flip the next proxy bit hot.
		:return: mo-dots (Dict-like) object containing the proxy's various parameters.
	"""
	global one_hot, proxies
	if not has_init:
		raise ActError("First, run actproxy.init() or actproxy.aioinit()")
	if not len(proxies):
		raise ActError('No proxies available. Check connection or ActProxy account.')
	hlen = len(one_hot) - 1
	hdex = 0
	try:
		hdex = one_hot.index(1)
	except ValueError:
		pass
	one_hot[hdex] = 0
	if hlen == hdex:
		one_hot[0] = 1
	else:
		one_hot[hdex + 1] = 1
	return to_data(proxies[hdex])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['actproxy']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp-socks>=0.5.5,<0.6.0',
 'aiohttp>=3.6.2,<4.0.0',
 'mo-dots>=3.93.20259,<4.0.0',
 'requests[socks]>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['setupgen = poetry.command_line:main']}

setup_kwargs = {
    'name': 'actproxy',
    'version': '0.1.3',
    'description': 'Sync & asyncio (Requests & AIOHTTP) proxy rotator + utils for actproxy API & services.',
    'long_description': '# actproxy\n\nPython package providing [actproxy.com](https://actproxy.com/aff.php?aff=30) API access and proxy rotation methods for requests (synchronous) and aiohttp\n(asyncio). Can also be used independently. Supports socks5, http/https, and ipv4/ipv6 as per actproxy\'s services.\n\n[![PyPI license](https://img.shields.io/pypi/l/ansicolortags.svg)](https://pypi.org/project/actproxy/)\n\n## Quick-Start (AIOHTTP)\n\n```python\nimport actproxy\nfrom aiohttp import ClientSession\n\n\nasync def main():\n    actproxy_api_key = "xxxxxxxxxxxxxxxxxxxxxxxx"\n    # Initialize API. Also returns your proxies.\n    await actproxy.aioinit(actproxy_api_key)\n    # Use a new AIOHTTP connector which rotates & uses the next proxy.\n    async with ClientSession(connector=actproxy.aiohttp_rotate()) as session:\n        url = "http://dummy.restapiexample.com/api/v1/employees"\n        async with session.get(url) as resp:\n            if resp.status == 200:\n                resp_json = await resp.json()\n                print(resp_json)\n```\n\n## Quick-Start (Requests)\n\n```python\nimport actproxy\nimport requests\n\nactproxy_api_key = "xxxxxxxxxxxxxxxxxxxxxxxx"\n# Initialize API. Also returns your proxies.\nactproxy.init(actproxy_api_key)\nurl = "http://dummy.restapiexample.com/api/v1/employees"\nresp = requests.get(url, proxies=actproxy.rotate())\nif resp.status_code == 200:\n    resp_json = resp.json()\n    print(resp_json)\n```\n\n## Methods\n\n`actproxy.aioinit(api_key)`: Fetches your proxies from ActProxy & returns them. Must be run before other aiohttp\nfunctions.\n\n`actproxy.init(api_key)`: Fetches your proxies from ActProxy & returns them. Must be run before other synchronous\nfunctions.\n\n`actproxy.aiohttp_rotate()`: Returns an aiohttp connector which uses the next proxy from your list.\n\n`actproxy.rotate()`: Returns the next proxy from your list. Return variable is suitable for use with requests[socks].\n\n`actproxy.random_proxy()`: Returns a random proxy from your list. Return variable is suitable for use with\nrequests[socks].\n\n`actproxy.aiohttp_random()`: Returns an aiohttp connector which uses uses a random proxy from your list.\n\n`actproxy.one_hot_proxy()`: Similar to rotate() but returns a single proxy dict/object for use in places other than\naiohttp or requests.\n\n## Changelog\n\n**0.1.3** - _9/29/2020_ : Minor fixes and addition of docstrings.\n\n**0.1.2** - _9/28/2020_ : Initial release version.',
    'author': 'TensorTom',
    'author_email': '14287229+TensorTom@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://actproxy.com/aff.php?aff=30',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.5,<4.0.0',
}


setup(**setup_kwargs)
